@@ -1,14 +1,17 @@
 from logging import Handler
 from collections import defaultdict
 from threading import currentThread
+from django.utils.log import AdminEmailHandler
+import json
+
 
 _request_log = defaultdict(list)
 _installed_middleware = False
-_should_log = False
+_should_log = defaultdict(lambda: False)
 
 
 def is_active():
-    return _should_log
+    return _should_log[currentThread()]
 
 
 def is_log_prepared():
@@ -16,7 +19,7 @@ def is_log_prepared():
 
 
 def get_request_log():
-    assert _installed_middleware, 'RequestCacheMiddleware not loaded'
+    assert _installed_middleware, 'RequestLogMiddleware not loaded'
     return _request_log[currentThread()]
 
 
@@ -26,7 +29,8 @@ class RequestHandler(Handler):
         Handler.__init__(self)
 
     def emit(self, record):
-        if _should_log:
+        if is_active():
+            self.format(record)
             get_request_log().append({
                 'message': record.message,
                 'pathname': record.pathname,
@@ -37,6 +41,12 @@ class RequestHandler(Handler):
             })
 
 
+class AdminJavascriptEmailHandler(AdminEmailHandler):
+
+    def format(self, record):
+        return 'Message: %s\nUser: %s\nClient Data: %s' % (record.getMessage(), record.user, json.dumps(record.client_data, indent=4, sort_keys=True))
+
+
 class RequestLogMiddleware(object):
 
     def __init__(self):
@@ -46,5 +56,5 @@ class RequestLogMiddleware(object):
     def process_request(self, request):
         global _should_log
         global _request_log
-        _should_log = 'debug' in request.GET and request.user.is_staff
+        _should_log[currentThread()] = 'debug' in request.GET and request.user.is_staff
         _request_log[currentThread()] = []
