@@ -6,44 +6,59 @@ from django.contrib.auth.models import User
 from django.dispatch import receiver
 from django.contrib.auth.signals import user_logged_in
 import logging
+from threading import currentThread
+
+_request_cache = {}
+_request_cache_initialized = False
+
 
 LOGGER = logging.getLogger(__name__)
 
 
+class ABMiddleware:
+
+    def process_request(self, request):
+        Experiment.objects.init_request(request)
+
+
 class ExperimentManager(models.Manager):
 
     def init_request(self, request):
-        if request.user.is_anonymous():
+        user = getattr(request, 'user', None)
+        if user is None or user.is_anonymous():
             self.clear_session(request.session)
+            self._init_request_cache(request)
             return
         if 'ab_experiment_values_modified' in request.session:
-            if request.user.id is None:
+            if user.id is None:
                 self.clear_session(request.session)
-            if request.user.id != request.session.get('ab_experiment_values_user'):
+            if user.id != request.session.get('ab_experiment_values_user'):
                 self.clear_session(request.session)
             if 'ab_experiment_reset' in request.GET:
                 self.clear_session(request.session)
         override = {}
-        if request.user.is_staff:
-            for key, value in request.GET.items():
+        if user.is_staff:
+            for key, value in list(request.GET.items()):
                 if key.startswith('ab_value_'):
                     override[key.replace('ab_value_', '')] = value
         if 'ab_experiment_values_modified' in request.session:
             saved_time = datetime.datetime.strptime(
                 request.session['ab_experiment_values_modified'], '%Y-%m-%d %H:%M:%S')
             if (datetime.datetime.now() - saved_time).total_seconds() < 15 * 60:
+                self._init_request_cache(request)
                 return
         if 'ab_experiment_values' not in request.session:
             request.session['ab_experiment_values'] = {}
-            request.session['ab_experiment_values_user'] = request.user.id
+            request.session['ab_experiment_values_user'] = user.id
         request.session['ab_experiment_values_modified'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
-        for k, v in override.iteritems():
+        for k, v in override.items():
             request.session['ab_experiment_values'][k] = v
-        for name, value in UserValue.objects.for_user(request.user).iteritems():
+        for name, value in UserValue.objects.for_user(user).items():
             if name in override:
                 continue
             request.session['ab_experiment_values'][name] = value
-        LOGGER.debug('initialized AB experiments for user %s: %s' % (str(request.user.id), str(request.session.get('ab_experiment_values', []))))
+        LOGGER.debug('initialized AB experiments for user %s: %s' % (str(user.id), str(request.session.get('ab_experiment_values', []))))
+        self._init_request_cache(request)
         return request
 
     def clear_session(self, session):
@@ -58,7 +73,7 @@ class ExperimentManager(models.Manager):
         total_prob = sum([probability for (probability, value) in values])
         if total_prob != 100:
             raise Exception('Total probability has to be equal to 100, it was ' + str(total_prob))
-        if default_value not in map(lambda (p, v): v, values):
+        if default_value not in [p_v[1] for p_v in values]:
             raise Exception('Default value %s is not present in values.' % default_value)
         experiment = Experiment(name=name, active=active)
         experiment.save()
@@ -75,8 +90,19 @@ class ExperimentManager(models.Manager):
                 return value['name']
         return default
 
-    def get_values(self, request):
-        return request.session.get('ab_experiment_values', {}).values()
+    def get_values(self, force=True):
+        if not force and not _request_cache_initialized:
+            return []
+        assert _request_cache_initialized, 'ABMiddleware is not loaded'
+        return _request_cache.get(currentThread(), [])
+
+    def _init_request_cache(self, request):
+        user = getattr(request, 'user', None)
+        global _request_cache_initialized
+        _request_cache_initialized = True
+        _request_cache[currentThread()] = list(Value.objects.filter(
+            id__in=[d['id'] for d in list(request.session.get('ab_experiment_values', {}).values())]))
+        LOGGER.debug('initialized request cache for AB experiments, user %s' % (str(None if user is None else user.id)))
 
 
 class Experiment(models.Model):
@@ -122,7 +148,6 @@ class Value(models.Model):
 
     class Meta:
         app_label = 'proso_ab'
-        unique_together = ('experiment', 'is_default')
 
     def to_json(self, nested=False):
         return {
@@ -156,7 +181,7 @@ class UserValueManager(models.Manager):
                 value = Value.objects.choose_value(experiment)
                 UserValue(user_id=user.id, value=value).save()
                 prepared[experiment.name] = value.to_json()
-        for experiment, default in defaults.iteritems():
+        for experiment, default in defaults.items():
             if experiment not in prepared:
                 prepared[experiment] = default.to_json()
         return prepared
