@@ -1,25 +1,11 @@
-from proso.django.response import render, render_json
-import json_enrich
+from proso.django.response import render_json
+from . import json_enrich
 import proso_common.json_enrich as common_json_enrich
 from django.shortcuts import get_object_or_404
-from models import Experiment
+from .models import Experiment
 from lazysignup.decorators import allow_lazy_user
 
 
-def home(request):
-    """
-    This module provides all necessary stuff to perform A/B experiments.
-
-    You can modify the behaviour of the module by the following GET parameter
-    (available on all pages):
-      ab_experiment_reset:
-        reset all cached values of A/B experiments for the current user
-      ab_value_(?P<name>[\w_]+):
-        set the value for of A/B experiment called <name>
-    """
-    return render(request, 'ab_home.html', {}, help_text=home.__doc__)
-
-
 def show_one(request, object_class, id):
     """
     Return object of the given type with the specified identifier.
@@ -53,14 +39,14 @@ def profile(request):
     """
     Return values for A/B experiments associated to the current user.
     """
-    objs = Experiment.objects.get_values(request)
+    objs = Experiment.objects.get_values()
     json = _to_json(request, objs)
     return render_json(request, json, template='ab_json.html', help_text=profile.__doc__)
 
 
 def _to_json(request, value):
     if isinstance(value, list):
-        json = map(lambda x: x if isinstance(x, dict) else x.to_json(), value)
+        json = [x if isinstance(x, dict) else x.to_json() for x in value]
     else:
         if isinstance(value, dict):
             json = value
